<?php

/**
 * @file
 * Data types, Actions and events of VirusTotal API Rules integration.
 *
 * @addtogroup rules
 * @{
 */

/**
 * Implements hook_rules_data_info().
 *
 * Declares meta-data about data types for VirusTotal Rules.
 */
function virustotal_rules_rules_data_info() {
  return array(
    // Data type that represents the API response.
    'virustotal_response' => array(
      'label' => t('VirusTotal API response'),
      'property info' => _virustotal_rules_rules_response_info(),
    ),
    // Data type that holds scan report contents.
    'virustotal_report' => array(
      'label' => t('VirusTotal Report'),
      'property info' => _virustotal_rules_rules_report_info(),
    ),
    // Data type used for scan reports to represent each AV result.
    'virustotal_finding' => array(
      'label' => t('VirusTotal AV Finding'),
      'wrap' => TRUE,
      'property info' => _virustotal_rules_rules_finding_info(),
    ),
  );
}

/**
 * Defines property info for VirusTotal API responses.
 * A data type that represents the API response.
 */
function _virustotal_rules_rules_response_info() {
  return array(
    'result' => array(
      'type' => 'integer',
      'label' => t('Result status code'),
      'options list' => 'virustotal_rules_response_status_codes',
    ),
    'scan_id' => array(
      'type' => 'text',
      'label' => ('Scan ID as returned by the file upload API'),
    ),
    'permalink' => array(
      'type' => 'text',
      'label' => ('Permanent link to report page'),
    ),
    'report' => array(
      'type' => 'virustotal_report',
      'label' => t('Report data'),
    ),
  );
}

/**
 * Defines property info for VirusTotal reports.
 * A data type that holds scan report contents.
 */
function _virustotal_rules_rules_report_info() {
  return array(
    'date' => array(
      'type' => 'date',
      'label' => t('Report date'),
    ),
    'findings' => array(
      'type' => 'list<virustotal_finding>',
      'label' => t('List of AV findings'),
    ),
  );
}

/**
 * Defines property info for VirusTotal AV findings.
 * A data type used for scan reports to represent each AV result.
 */
function _virustotal_rules_rules_finding_info() {
  return array(
    'av' => array(
      'type' => 'text',
      'label' => ('Title of AV product'),
    ),
    'result' => array(
      'type' => 'text',
      'label' => ('Scan result'),
    ),
  );
}

/**
 * Select list of response status codes.
 */
function virustotal_rules_response_status_codes() {
  // Inlcude VirusTotal API class, otherwise constants won't be defined.
  include_once drupal_get_path('module', 'virustotal') . '/virustotal.class.php';
  return array(
    VIRUSTOTAL_RESULT_OK => t('OK'),
    VIRUSTOTAL_RESULT_NOT_FOUND => t('Not found'),
    VIRUSTOTAL_RESULT_INVALID_KEY => t('Invalid key'),
    VIRUSTOTAL_RESULT_EXCEED_QUOTA => t('Exceed quota'),
    VIRUSTOTAL_RESULT_HTTP_ERROR => t('HTTP Error'),
  );
}

/**
 * Implements hook_rules_action_info().
 *
 * Declares meta-data about actions for VirusTotal Rules.
 */
function virustotal_rules_rules_action_info() {
  $actions = array();

  // Action to send a file to VirusTotal service.
  $actions['virustotal_rules_action_send_file'] = array(
    'label' => t('Send a File'),
    'group' => t('VirusTotal'),
    'parameter' => array(
      'file' => array(
        'type' => 'file',
        'label' => t('File Object'),
        'description' => t('File Object of the local file you want to send'),
      ),
    ),
    'provides' => array(
      'response' => array(
        'type' => 'virustotal_response',
        'label' => t('File scan response'),
      ),
    ),
  );

  // Action to send an URL to VirusTotal service.
  $actions['virustotal_rules_action_send_url'] = array(
    'label' => t('Send an URL'),
    'group' => t('VirusTotal'),
    'parameter' => array(
      'url' => array(
        'type' => 'text',
        'label' => t('URL'),
        'description' => t('The URL that should be scanned'),
      ),
    ),
    'provides' => array(
      'response' => array(
        'type' => 'virustotal_response',
        'label' => t('URL scan response'),
      ),
    ),
  );

  // Action to retrieve a file report from VirusTotal service.
  $actions['virustotal_rules_action_get_file_report'] = array(
    'label' => t('Retrieve a file report'),
    'group' => t('VirusTotal'),
    'parameter' => array(
      'resource' => array(
        'type' => 'text',
        'label' => t('Resource'),
        'description' => t('A md5/sha1/sha256 hash will retrieve the most recent report on a given sample. You may also specify a permalink identifier (sha256-timestamp (Scan ID) as returned by the file upload API) to access a specific report.'),
      ),
    ),
    'provides' => array(
      'response' => array(
        'type' => 'virustotal_response',
        'label' => t('File report response'),
      ),
    ),
  );

  // Action to retrieve an URL report from VirusTotal service.
  $actions['virustotal_rules_action_get_url_report'] = array(
    'label' => t('Retrieve an URL report'),
    'group' => t('VirusTotal'),
    'parameter' => array(
      'resource' => array(
        'type' => 'text',
        'label' => t('Resource'),
        'description' => t('A URL will retrieve the most recent report on the given URL. You may also specify a permalink identifier (md5-timestamp as returned by the URL submission API) to access a specific report.'),
      ),
    ),
    'provides' => array(
      'response' => array(
        'type' => 'virustotal_response',
        'label' => t('URL report response'),
      ),
    ),
  );

  // Action to create a comment on a file or URL report.
  $actions['virustotal_rules_action_create_comment'] = array(
    'label' => t('Comment a report'),
    'group' => t('VirusTotal'),
    'parameter' => array(
      'resource' => array(
        'type' => 'text',
        'label' => t('Resource'),
        'description' => t('Either a md5/sha1/sha256 hash of the file you want to review or the URL itself that you want to comment on.'),
      ),
      'comment' => array(
        'type' => 'text',
        'label' => t('Comment'),
        'description' => t('The actual review, you can tag it using the "#" twitter-like syntax (e.g. #disinfection #zbot) and reference users using the "@" syntax (e.g. @EmilianoMartinez).'),
      ),
      'type' => array(
        'type' => 'text',
        'label' => t('Resource type'),
        'description' => t("String containing 'file' or 'url' depending on what you want to comment."),
        'options list' => 'virustotal_rules_comment_tag_list',
      ),
      'tags' => array(
        'type' => 'list<text>',
        'label' => t('Tags'),
        'description' => t('This parameter is optional. It should be a list containing text of standard file or URL tags. The standard file tags are: goodware, malware, spamattachmentorlink, p2pdownload, impropagating, networkworm, drivebydownload. The standard URL tags are: malicious, benign, malwaredownload, phishingsite, browserexploit, spamlink.'),
        'optional' => TRUE,
      ),
    ),
    'provides' => array(
      'response' => array(
        'type' => 'virustotal_response',
        'label' => t('URL report response'),
      ),
    ),
  );

  return $actions;
}

/**
 * The action function for the 'virustotal_rules_action_send_file'.
 * An action to send a file to VirusTotal service.
 */
function virustotal_rules_action_send_file($file) {
  // Submit the file for scanning.
  $vt = new VirusTotal();
  $response = $vt->scanFile($file);

  // Provide response data as expected to rules.
  return array(
    'response' => array(
      'result' => $response['result'],
      'scan_id' => $response['scan_id'],
    ),
  );
}

/**
 * The action function for the 'virustotal_rules_action_send_url'.
 * An action to send an URL to VirusTotal service.
 */
function virustotal_rules_action_send_url($url) {
  // Submit the URL for scanning.
  $vt = new VirusTotal();
  $response = $vt->scanUrl($url);

  // Provide response data as expected to rules.
  return array(
    'response' => array(
      'result' => $response['result'],
      'scan_id' => $response['scan_id'],
    ),
  );
}

/**
 * The action function for the 'virustotal_rules_action_get_file_report'.
 * An action to retrieve a file report from VirusTotal service.
 */
function virustotal_rules_action_get_file_report($resource) {
  // Request a file report.
  $vt = new VirusTotal();
  $response = $vt->getFileReport($resource);

  // Assemble list of findings (data type).
  $findings = array();
  foreach ($response['report'][1] as $av => $result) {
    $findings[] = array(
      'av' => $av,
      'result' => $result,
    );
  }

  // Provide response data to rules.
  return array(
    'response' => array(
      'result' => $response['result'],
      'permalink' => $response['permalink'],
      'report' => array(
        'date' => strtotime($response['report'][0]),
        'findings' => $findings,
      ),
    ),
  );
}

/**
 * The action function for the 'virustotal_rules_action_get_url_report'.
 * An action to retrieve an URL report from VirusTotal service.
 */
function virustotal_rules_action_get_url_report($resource) {
  // Request an URL report.
  $vt = new VirusTotal();
  $response = $vt->getUrlReport($resource);

  // Assemble list of findings (data type).
  $findings = array();
  foreach ($response['report'][1] as $av => $result) {
    $findings[] = array(
      'av' => $av,
      'result' => $result,
    );
  }

  // Provide response data to rules.
  return array(
    'response' => array(
      'result' => $response['result'],
      'report' => array(
        'date' => strtotime($response['report'][0]),
        'findings' => $findings,
      ),
    ),
  );
}

/**
 * The action function for the 'virustotal_rules_action_create_comment'.
 */
function virustotal_rules_action_create_comment($resource, $comment, $type, $tags = array()) {
  // Create a comment on a file / URL report.
  $vt = new VirusTotal();
  $response = $vt->makeComment($type, $resource, $comment, $tags);

  // Provide response data to rules.
  return array(
    'response' => array(
    'result' => $response['result'],
  ));
}

/**
 * Static select list of report types.
 */
function virustotal_rules_comment_tag_list() {
  return array(
    'file' => t('File'),
    'url' => t('URL'),
  );
}

/**
 * Implements hook_rules_event_info().
 *
 * Declares meta-data about events for VirusTotal Rules.
 */
function virustotal_rules_rules_event_info() {
  $events = array();

  // Event that triggers before an API request is send.
  $events['virustotal_rules_event_pre_request'] = array(
    'label' => t('Before sending a request'),
    'group' => t('VirusTotal'),
    'variables' => array(
      'function' => array(
        'type' => 'text',
        'label' => t('API function'),
        // 'options list' => 'virustotal_rules_api_functions',
      ),
    ),
  );

  // Event that triggers after an API request was send.
  $events['virustotal_rules_event_post_request'] = array(
    'label' => t('After sending a request'),
    'group' => t('VirusTotal'),
    'variables' => array(
     /* @TODO
      * This needs a handler implementation so values can be get and set.
      'response' => array(
        'type' => 'virustotal_response',
        'label' => t('API response'),
      ),
      */
      'function' => array(
        'type' => 'text',
        'label' => t('API function'),
        // 'options list' => 'virustotal_rules_api_functions',
      ),
    ),
  );

  return $events;
}

/**
 * Static select list of API functions.
 */
/* @TODO
 * Options list of API functions don't work on pre and post request events.
 * Need more information on how to define them in events or find the bug.
function virustotal_rules_api_functions() {
  // Inlcude VirusTotal API class, otherwise constants won't be defined.
include_once drupal_get_path('module', 'virustotal') . '/virustotal.class.php';
  return array(
    VIRUSTOTAL_FUNC_SCAN_FILE => t('Scan file'),
    VIRUSTOTAL_FUNC_GET_FILE_REPORT => t('Get file report'),
    VIRUSTOTAL_FUNC_SCAN_URL => t('Scan URL'),
    VIRUSTOTAL_FUNC_GET_URL_REPORT => t('Get url report'),
    VIRUSTOTAL_FUNC_MAKE_COMMENT => t('Make comment'),
  );
}
 */

/**
 * @}
 */
